/*
 * Copyright 2001-2003 Sun Microsystems, Inc.  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
 * CA 95054 USA or visit www.sun.com if you need additional information or
 * have any questions.
 */

#ifndef XRGlyphCache_h_Included
#define XRGlyphCache_h_Included

#include "awt_p.h"
#include "awt_GraphicsEnv.h"

#include <X11/extensions/Xrender.h>
#include "jni.h"
#include "fontscalerdefs.h"

typedef struct _XRGlyphCacheInfo XRGlyphCacheInfo;
typedef struct _XRCacheCellInfo XRCacheCellInfo;

/* Cell-Cache structure*/
struct _XRGlyphCacheInfo {
    XRCacheCellInfo *cells;
    jint searchPosition;
    jint usedCells;
    jint cellCapacity;
    jint pixelsUsed;
    GlyphSet glyphSet;
};

/* CacheCellInfo structure, 
*  holds a cached GlyphInfo and
*  other per-glpyh information.
*/
struct _XRCacheCellInfo {
    XRGlyphCacheInfo *cacheInfo;
    GlyphInfo *glyphInfo;
    unsigned short glyphID;
    jint timesRendered;
    jint width;
    jint height;
    jint xOff;
    jint yOff;
    jboolean pinned;
};

#ifndef HEADLESS
/*Initialized the GlyphCache structure*/
XRGlyphCacheInfo* XRGlyphCache_Init(XRenderPictFormat* glyphSetPictFormat);

/*Add a list of GlyphInfo's to the cache, returns false of not succeeded.*/
jboolean XRGlyphCache_CacheGlyphs(XRGlyphCacheInfo *cache, jlong *glyphPtrs, jint glyphCount, jint *ids);

/* Frees entries which have not been used a lot, and enlarges cache if nescessary */
static jboolean cleanCache(XRGlyphCacheInfo *cache);

/* Adds a single glyph to the cache and uploads it to the X-Server*/
static jboolean uploadGlyph(XRGlyphCacheInfo *cache, GlyphInfo *jginfo, XRCacheCellInfo *cacheCell);
#endif

#endif /* XRGlyphCache_h_Included */
